<?php
add_filter('manage_edit-product_columns', 'mrt_redirect_add_product_columns');
add_action('manage_product_posts_custom_column', 'mrt_redirect_product_column_content', 10, 2);
add_filter('manage_edit-product_sortable_columns', 'mrt_redirect_sortable_product_columns');

add_action('admin_head', 'mrt_redirect_screen_options_js');

function mrt_redirect_add_product_columns($columns)
{
    $columns['mrt_redirect_column'] = __('MRT Redirect', 'woocommerce');
    $columns['mrt_sync_column'] = __('MRT Sync', 'woocommerce');
    return $columns;
}

function mrt_redirect_product_column_content($column, $post_id)
{
    if ($column === 'mrt_redirect_column') {
        if (get_post_meta($post_id, '_mrtarget_purchase_activated', true) == 'yes') {
            echo '<div class="mrt_redirect_column_icon">';
            echo get_post_meta($post_id, '_mrtarget_purchase_activated', true);
            echo '</div>';
        }
    }
    if ($column === 'mrt_sync_column') {
        if (get_post_meta($post_id, '_mrtarget_sync_activated', true) == 'yes') {
            echo '<div class="mrt_sync_column_icon">';
            echo get_post_meta($post_id, '_mrtarget_sync_activated', true);
            echo '</div>';
        }
    }
}

function mrt_redirect_sortable_product_columns($columns)
{
    $columns['mrt_redirect_column'] = 'mrt_redirect_column';
    $columns['mrt_sync_column'] = 'mrt_sync_column';
    return $columns;
}

function mrt_redirect_screen_options_js()
{
    if (get_current_screen()->id !== 'edit-product') {
        return;
    }
?>
    <script type="text/javascript">
        jQuery(document).ready(function($) {
            $('#adv-settings').append($('#mrt_redirect_column-hide').closest('fieldset'));
            $('#adv-settings').append($('#mrt_sync_column-hide').closest('fieldset'));
        });
    </script>
    <style>
        table.fixed {
            table-layout: auto;
        }

        .mrt_redirect_column,
        .mrt_sync_column {
            text-indent: -9999px;
            position: relative;
            margin-top: 5px;
            /* height: 1em;
            width: 1em;
            text-align: left; */
            font-size: 2.6em !important;
        }

        .mrt_redirect_column_icon::before,
        .mrt_sync_column_icon::before {
            content: "\f15e";
            font-family: Dashicons;
            font-weight: 400;
            font-variant: normal;
            text-transform: none;
            line-height: 1;
            -webkit-font-smoothing: antialiased;
            margin: 0;
            text-indent: 0;
            position: absolute;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            text-align: center;
            color: #ff6d10;
        }
    </style>
    <?php
}

add_action('restrict_manage_posts', 'mrt_redirect_add_filter');
add_filter('parse_query', 'mrt_redirect_filter_query');

function mrt_redirect_add_filter()
{
    global $typenow;

    if ($typenow == 'product') {
    ?>
        <select name="mrt_redirect_column_filter" id="mrt_redirect_column_filter">
            <option value=""><?php _e('All MRT Redirect Values', 'woocommerce'); ?></option>
            <option value="yes" <?php selected(isset($_GET['mrt_redirect_column_filter']) ? $_GET['mrt_redirect_column_filter'] : '', 'yes'); ?>><?php _e('Redirected', 'woocommerce'); ?></option>
        </select>
        <select name="mrt_sync_column_filter" id="mrt_sync_column_filter">
            <option value=""><?php _e('All MRT Sync Values', 'woocommerce'); ?></option>
            <option value="yes" <?php selected(isset($_GET['mrt_sync_column_filter']) ? $_GET['mrt_sync_column_filter'] : '', 'yes'); ?>><?php _e('Synced', 'woocommerce'); ?></option>
        </select>
<?php
    }
}

function mrt_redirect_filter_query($query)
{
    global $pagenow, $typenow;

    if ($pagenow == 'edit.php' && $typenow == 'product' && isset($_GET['mrt_redirect_column_filter']) && $_GET['mrt_redirect_column_filter'] != '') {
        $query->query_vars['meta_query'][] = array(
            'key'     => '_mrtarget_purchase_activated',
            'value'   => $_GET['mrt_redirect_column_filter'],
            'compare' => '='
        );
    }
    if ($pagenow == 'edit.php' && $typenow == 'product' && isset($_GET['mrt_sync_column_filter']) && $_GET['mrt_sync_column_filter'] != '') {
        $query->query_vars['meta_query'][] = array(
            'key'     => '_mrtarget_sync_activated',
            'value'   => $_GET['mrt_sync_column_filter'],
            'compare' => '='
        );
    }
}
