<?php

/**
 * mrtarget sync
 */

function get_external_product_data($sku)
{
    $transient_key = 'external_product_data_' . $sku;
    $cached_data = get_transient($transient_key);

    if ($cached_data !== false) {
        return $cached_data;
    }

    $api_url = MR_TARGET_SYNC_URL . $sku;

    $args = array(
        //'body'        => json_encode($data),
        'headers'     => array(
            'Content-Type' => 'application/json',
        ),
        //'method'      => 'POST',
        //'data_format' => 'body',
        'sslverify'   => false,
    );
    $response = wp_remote_get($api_url, $args);

    if (is_wp_error($response)) {
        return false;
    }

    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body);

    //set_transient($transient_key, $data, HOUR_IN_SECONDS);
    set_transient($transient_key, $data, MR_TARGET_API_AND_CACHE_TIME_IN_SECONDS);

    return $data;
}

add_action('save_post', 'update_product_data', 10, 3);

function update_product_data($post_id, $post, $update)
{
    if (get_post_type($post_id) === 'product') {
        $mrtarget_sync_activated = isset($_POST['_mrtarget_sync_activated']) ? 'yes' : 'no';
        if (!empty($mrtarget_sync_activated)) {
            if ($mrtarget_sync_activated == 'yes') {
                $sku = get_post_meta($post_id, '_sku', true);
                if (!empty($sku)) {
                    remove_action('save_post', 'update_product_data');
                    $external_data = get_external_product_data($sku);

                    //error_log(print_r($external_data, true) . PHP_EOL, 3, wp_get_upload_dir()['basedir'] . '/external_data.log');
                    if (isset($external_data->sku)) {
                        if (isset($external_data->name)) {
                            wp_update_post(array(
                                'ID' => $post_id,
                                'post_title' => $external_data->name
                            ));
                        }
                        if (isset($external_data->description)) {
                            wp_update_post(array(
                                'ID' => $post_id,
                                'post_content' => $external_data->description
                            ));
                        }
                        if (isset($external_data->price)) {
                            update_post_meta($post_id, '_price', $external_data->price);
                        }
                        if (isset($external_data->regular_price)) {
                            update_post_meta($post_id, '_regular_price', $external_data->regular_price);
                        }
                        if (isset($external_data->sale_price)) {
                            update_post_meta($post_id, '_sale_price', $external_data->sale_price);
                        }
                        if (isset($external_data->stock_quantity)) {
                            update_post_meta($post_id, '_stock', $external_data->stock_quantity);
                        }
                        if (isset($external_data->weight)) {
                            update_post_meta($post_id, '_weight', $external_data->weight);
                        }
                        if (isset($external_data->length)) {
                            update_post_meta($post_id, '_length', $external_data->length);
                        }
                        if (isset($external_data->width)) {
                            update_post_meta($post_id, '_width', $external_data->width);
                        }
                        if (isset($external_data->height)) {
                            update_post_meta($post_id, '_height', $external_data->height);
                        }
                        if (isset($external_data->categories)) {
                            $categories = [];
                            foreach ($external_data->categories as $category) {
                                $term = get_term_by('name', $category, 'product_cat');
                                if ($term) {
                                    $categories[] = $term->term_id;
                                }
                            }
                            wp_set_post_terms($post_id, $categories, 'product_cat');
                        }
                        if (isset($external_data->tags)) {
                            $tags = [];
                            foreach ($external_data->tags as $tag) {
                                $term = get_term_by('name', $tag, 'product_tag');
                                if ($term) {
                                    $tags[] = $term->term_id;
                                }
                            }
                            wp_set_post_terms($post_id, $tags, 'product_tag');
                        }
                        if (isset($external_data->featured_image)) {
                            $image_url = $external_data->featured_image;
                            // Download the image from the URL
                            $image_data = file_get_contents($image_url);

                            // If the image data is retrieved successfully
                            if ($image_data !== false) {
                                // Get the upload directory
                                $upload_dir = wp_upload_dir();
                                $filename = basename($image_url);
                                $file_path = $upload_dir['path'] . '/' . $filename;

                                // Save the image to the upload directory
                                file_put_contents($file_path, $image_data);

                                // Check the file type and prepare the attachment data
                                $wp_filetype = wp_check_filetype($filename, null);
                                $attachment = array(
                                    'post_mime_type' => $wp_filetype['type'],
                                    'post_title' => sanitize_file_name($filename),
                                    'post_content' => '',
                                    'post_status' => 'inherit'
                                );

                                // Insert the attachment into the WordPress media library
                                $attach_id = wp_insert_attachment($attachment, $file_path);

                                // Include the necessary WordPress file for generating attachment metadata
                                require_once(ABSPATH . 'wp-admin/includes/image.php');

                                // Generate and update the attachment metadata
                                $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
                                wp_update_attachment_metadata($attach_id, $attach_data);

                                // Set the image as the product's featured image (thumbnail)
                                set_post_thumbnail($post_id, $attach_id);
                            }
                        }
                        if (isset($external_data->gallery_images)) {
                            $image_urls = $external_data->gallery_images;
                            $gallery_image_ids = [];

                            foreach ($image_urls as $image_url) {
                                $image_data = file_get_contents($image_url);

                                if ($image_data !== false) {
                                    $upload_dir = wp_upload_dir();
                                    $filename = basename($image_url);
                                    $file_path = $upload_dir['path'] . '/' . $filename;

                                    file_put_contents($file_path, $image_data);

                                    $wp_filetype = wp_check_filetype($filename, null);
                                    $attachment = array(
                                        'post_mime_type' => $wp_filetype['type'],
                                        'post_title' => sanitize_file_name($filename),
                                        'post_content' => '',
                                        'post_status' => 'inherit'
                                    );

                                    $attach_id = wp_insert_attachment($attachment, $file_path);

                                    require_once(ABSPATH . 'wp-admin/includes/image.php');
                                    $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
                                    wp_update_attachment_metadata($attach_id, $attach_data);

                                    // Add the image ID to the gallery array
                                    $gallery_image_ids[] = $attach_id;
                                }
                            }

                            // Update the product gallery meta field with the image IDs
                            if (!empty($gallery_image_ids)) {
                                update_post_meta($post_id, '_product_image_gallery', implode(',', $gallery_image_ids));
                            }
                        }
                        if (!empty($external_data->attributes)) {
                            $product_attributes = array();
                            foreach ($external_data->attributes as $attribute) {
                                //$attribute_name = wc_attribute_taxonomy_name($attribute['name']);
                                //$attribute_name = $attribute['name'];
                                $attribute_name = wc_sanitize_taxonomy_name($attribute->name);
                                $attribute_label = $attribute->name;

                                if (!taxonomy_exists($attribute_name)) {
                                    register_taxonomy(
                                        $attribute_name,
                                        'product',
                                        array(
                                            'label'        => $attribute_label,
                                            'public'       => true,
                                            'show_ui'      => false,
                                            'query_var'    => true,
                                            'rewrite'      => false,
                                            'hierarchical' => true,
                                        )
                                    );
                                }

                                if ($attribute->is_taxonomy) {
                                    wp_set_object_terms($post_id, $attribute->value, $attribute_name);
                                    $product_attributes[$attribute_name] = array(
                                        'name' => $attribute_label,
                                        'value' => '',
                                        'position' => 0,
                                        'is_visible' => 1,
                                        'is_variation' => 1,
                                        'is_taxonomy' => 1,
                                    );
                                } else {
                                    $product_attributes[$attribute_name] = array(
                                        'name' => $attribute_label,
                                        'value' => implode(' | ', $attribute->value),
                                        'position' => 0,
                                        'is_visible' => 1,
                                        'is_variation' => 1,
                                        'is_taxonomy' => 0,
                                    );
                                }
                            }

                            update_post_meta($post_id, '_product_attributes', $product_attributes);
                        }


                        if (!empty($external_data->variations)) {
                            wp_set_object_terms($post_id, 'variable', 'product_type');
                            foreach ($external_data->variations as $variation_data) {
                                $variation_post = array(
                                    'post_title'  => $external_data->name . ' - Variation',
                                    'post_name'   => 'product-' . $post_id . '-variation',
                                    'post_status' => 'publish',
                                    'post_parent' => $post_id,
                                    'post_type'   => 'product_variation',
                                    'guid'        => home_url('/?product_variation=' . $post_id),
                                );

                                $variation_id = wp_insert_post($variation_post);

                                update_post_meta($variation_id, '_sku', $variation_data->sku);
                                update_post_meta($variation_id, '_price', $variation_data->price);
                                update_post_meta($variation_id, '_regular_price', $variation_data->regular_price);
                                update_post_meta($variation_id, '_sale_price', $variation_data->sale_price);
                                update_post_meta($variation_id, '_stock', $variation_data->stock_quantity);

                                foreach ($variation_data->attributes as $attribute_name => $attribute_value) {
                                    //$taxonomy = wc_attribute_taxonomy_name($attribute_name);
                                    $taxonomy = $attribute_name;
                                    update_post_meta($variation_id, 'attribute_' . $taxonomy, $attribute_value);
                                }
                            }
                        }
                        update_post_meta($post_id, '_mrtarget_sync_activated', 'yes');
                    }
                    add_action('save_post', 'update_product_data', 10, 3);
                }
            } else {
                update_post_meta($post_id, '_mrtarget_sync_activated', 'no');
            }
        }
    }
}

// Add a 30-minute interval to the cron
function custom_cron_schedules($schedules)
{
    $schedules['every_thirty_minutes'] = array(
        'interval' => MR_TARGET_API_AND_CACHE_TIME_IN_SECONDS, // Time in seconds (1800 seconds = 30 minutes)
        'display' => __('Every Thirty Minutes')
    );
    return $schedules;
}
add_filter('cron_schedules', 'custom_cron_schedules');

// Schedule the event if it is not already scheduled
if (!wp_next_scheduled('sync_external_product_data_event')) {
    wp_schedule_event(time(), 'every_thirty_minutes', 'sync_external_product_data_event');
}

// Hook into the event
add_action('sync_external_product_data_event', 'sync_external_product_data');

// Data synchronization function
function sync_external_product_data()
{
    // Get all products for synchronization
    $query = new WP_Query(array(
        'post_type' => 'product',
        'meta_key' => '_mrtarget_sync_activated',
        'meta_value' => 'yes',
        'posts_per_page' => -1
    ));

    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            $post_id = get_the_ID();
            $sku = get_post_meta($post_id, '_sku', true);
            if (!empty($sku)) {
                $external_data = get_external_product_data($sku);
                if (isset($external_data->sku)) {
                    // Update product data
                    if (isset($external_data->name)) {
                        wp_update_post(array(
                            'ID' => $post_id,
                            'post_title' => $external_data->name
                        ));
                    }
                    if (isset($external_data->description)) {
                        wp_update_post(array(
                            'ID' => $post_id,
                            'post_content' => $external_data->description
                        ));
                    }
                    if (isset($external_data->price)) {
                        update_post_meta($post_id, '_price', $external_data->price);
                    }
                    if (isset($external_data->regular_price)) {
                        update_post_meta($post_id, '_regular_price', $external_data->regular_price);
                    }
                    if (isset($external_data->sale_price)) {
                        update_post_meta($post_id, '_sale_price', $external_data->sale_price);
                    }
                    if (isset($external_data->stock_quantity)) {
                        update_post_meta($post_id, '_stock', $external_data->stock_quantity);
                    }
                    if (isset($external_data->weight)) {
                        update_post_meta($post_id, '_weight', $external_data->weight);
                    }
                    if (isset($external_data->length)) {
                        update_post_meta($post_id, '_length', $external_data->length);
                    }
                    if (isset($external_data->width)) {
                        update_post_meta($post_id, '_width', $external_data->width);
                    }
                    if (isset($external_data->height)) {
                        update_post_meta($post_id, '_height', $external_data->height);
                    }
                    if (isset($external_data->categories)) {
                        $categories = [];
                        foreach ($external_data->categories as $category) {
                            $term = get_term_by('name', $category, 'product_cat');
                            if ($term) {
                                $categories[] = $term->term_id;
                            }
                        }
                        wp_set_post_terms($post_id, $categories, 'product_cat');
                    }
                    if (isset($external_data->tags)) {
                        $tags = [];
                        foreach ($external_data->tags as $tag) {
                            $term = get_term_by('name', $tag, 'product_tag');
                            if ($term) {
                                $tags[] = $term->term_id;
                            }
                        }
                        wp_set_post_terms($post_id, $tags, 'product_tag');
                    }
                    if (isset($external_data->featured_image)) {
                        $image_url = $external_data->featured_image;
                        // Download the image from the URL
                        $image_data = file_get_contents($image_url);

                        // If the image data is retrieved successfully
                        if ($image_data !== false) {
                            // Get the upload directory
                            $upload_dir = wp_upload_dir();
                            $filename = basename($image_url);
                            $file_path = $upload_dir['path'] . '/' . $filename;

                            // Save the image to the upload directory
                            file_put_contents($file_path, $image_data);

                            // Check the file type and prepare the attachment data
                            $wp_filetype = wp_check_filetype($filename, null);
                            $attachment = array(
                                'post_mime_type' => $wp_filetype['type'],
                                'post_title' => sanitize_file_name($filename),
                                'post_content' => '',
                                'post_status' => 'inherit'
                            );

                            // Insert the attachment into the WordPress media library
                            $attach_id = wp_insert_attachment($attachment, $file_path);

                            // Include the necessary WordPress file for generating attachment metadata
                            require_once(ABSPATH . 'wp-admin/includes/image.php');

                            // Generate and update the attachment metadata
                            $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
                            wp_update_attachment_metadata($attach_id, $attach_data);

                            // Set the image as the product's featured image (thumbnail)
                            set_post_thumbnail($post_id, $attach_id);
                        }
                    }
                    if (isset($external_data->gallery_images)) {
                        $image_urls = $external_data->gallery_images;
                        $gallery_image_ids = [];

                        foreach ($image_urls as $image_url) {
                            $image_data = file_get_contents($image_url);

                            if ($image_data !== false) {
                                $upload_dir = wp_upload_dir();
                                $filename = basename($image_url);
                                $file_path = $upload_dir['path'] . '/' . $filename;

                                file_put_contents($file_path, $image_data);

                                $wp_filetype = wp_check_filetype($filename, null);
                                $attachment = array(
                                    'post_mime_type' => $wp_filetype['type'],
                                    'post_title' => sanitize_file_name($filename),
                                    'post_content' => '',
                                    'post_status' => 'inherit'
                                );

                                $attach_id = wp_insert_attachment($attachment, $file_path);

                                require_once(ABSPATH . 'wp-admin/includes/image.php');
                                $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
                                wp_update_attachment_metadata($attach_id, $attach_data);

                                // Add the image ID to the gallery array
                                $gallery_image_ids[] = $attach_id;
                            }
                        }

                        // Update the product gallery meta field with the image IDs
                        if (!empty($gallery_image_ids)) {
                            update_post_meta($post_id, '_product_image_gallery', implode(',', $gallery_image_ids));
                        }
                    }
                    if (!empty($external_data->attributes)) {
                        $product_attributes = array();
                        foreach ($external_data->attributes as $attribute) {
                            //$attribute_name = wc_attribute_taxonomy_name($attribute['name']);
                            //$attribute_name = $attribute['name'];
                            $attribute_name = wc_sanitize_taxonomy_name($attribute->name);
                            $attribute_label = $attribute->name;

                            if (!taxonomy_exists($attribute_name)) {
                                register_taxonomy(
                                    $attribute_name,
                                    'product',
                                    array(
                                        'label'        => $attribute_label,
                                        'public'       => true,
                                        'show_ui'      => false,
                                        'query_var'    => true,
                                        'rewrite'      => false,
                                        'hierarchical' => true,
                                    )
                                );
                            }

                            if ($attribute->is_taxonomy) {
                                wp_set_object_terms($post_id, $attribute->value, $attribute_name);
                                $product_attributes[$attribute_name] = array(
                                    'name' => $attribute_label,
                                    'value' => '',
                                    'position' => 0,
                                    'is_visible' => 1,
                                    'is_variation' => 1,
                                    'is_taxonomy' => 1,
                                );
                            } else {
                                $product_attributes[$attribute_name] = array(
                                    'name' => $attribute_label,
                                    'value' => implode(' | ', $attribute->value),
                                    'position' => 0,
                                    'is_visible' => 1,
                                    'is_variation' => 1,
                                    'is_taxonomy' => 0,
                                );
                            }
                        }

                        update_post_meta($post_id, '_product_attributes', $product_attributes);
                    }


                    if (!empty($external_data->variations)) {
                        wp_set_object_terms($post_id, 'variable', 'product_type');
                        foreach ($external_data->variations as $variation_data) {
                            $variation_post = array(
                                'post_title'  => $external_data->name . ' - Variation',
                                'post_name'   => 'product-' . $post_id . '-variation',
                                'post_status' => 'publish',
                                'post_parent' => $post_id,
                                'post_type'   => 'product_variation',
                                'guid'        => home_url('/?product_variation=' . $post_id),
                            );

                            $variation_id = wp_insert_post($variation_post);

                            update_post_meta($variation_id, '_sku', $variation_data->sku);
                            update_post_meta($variation_id, '_price', $variation_data->price);
                            update_post_meta($variation_id, '_regular_price', $variation_data->regular_price);
                            update_post_meta($variation_id, '_sale_price', $variation_data->sale_price);
                            update_post_meta($variation_id, '_stock', $variation_data->stock_quantity);

                            foreach ($variation_data->attributes as $attribute_name => $attribute_value) {
                                //$taxonomy = wc_attribute_taxonomy_name($attribute_name);
                                $taxonomy = $attribute_name;
                                update_post_meta($variation_id, 'attribute_' . $taxonomy, $attribute_value);
                            }
                        }
                    }
                }
                update_post_meta($post_id, '_mrtarget_sync_activated', 'yes');
            }
        }
        wp_reset_postdata();
    }
}

// Deactivate cron event on plugin deactivation
function my_deactivation()
{
    wp_clear_scheduled_hook('sync_external_product_data_event');
}
register_deactivation_hook(__FILE__, 'my_deactivation');
