<?php
add_action('admin_enqueue_scripts', 'mr_target_client_enqueue_styles');
add_action('wp_enqueue_scripts', 'mr_target_client_enqueue_styles');

function mr_target_client_enqueue_styles()
{
    wp_enqueue_style('mr-target-plugin-client-style', plugins_url('style.css', __FILE__), array(), time());
}
include MR_TARGET_PLUGIN_DIR . 'includes/admin-product-options.php';
include MR_TARGET_PLUGIN_DIR . 'includes/templates-rewrite.php';
include MR_TARGET_PLUGIN_DIR . 'includes/all-products-table.php';
include MR_TARGET_PLUGIN_DIR . 'includes/sync.php';

/**
 * sku options page
 */
add_action('admin_menu', 'custom_add_client_menu_page');

function custom_add_client_menu_page()
{
    add_menu_page(
        __('MrTarget Products', 'mrtargettextdomain'),
        __('MrTarget Products', 'mrtargettextdomain'),
        'manage_options',
        'mrtarget-client-page',
        'mrtarget-client-page-content',
        'dashicons-admin-generic',
        6
    );

    add_submenu_page(
        'mrtarget-client-page',
        __('Products Catalog', 'mrtargettextdomain'),
        __('Products Catalog', 'mrtargettextdomain'),
        'manage_options',
        'mr-target-product-catalog',
        'mrtarget_product_catalog_options_content'
    );

    add_submenu_page(
        'mrtarget-client-page',
        __('Options', 'mrtargettextdomain'),
        __('Options', 'mrtargettextdomain'),
        'manage_options',
        'mr-target-client-options',
        'mrtarget_client_options'
    );
}

function mrtarget_client_options()
{
    include plugin_dir_path(__FILE__) . 'options/options_content_template.php';
}

function mrtarget_product_catalog_options_content()
{
    include plugin_dir_path(__FILE__) . 'options/product_catalog_options_content_template.php';
}


add_action('admin_init', 'mrt_target_client_custom_redirect_to_first_subpage');
function mrt_target_client_custom_redirect_to_first_subpage()
{
    global $pagenow;
    if ($pagenow == 'admin.php' && isset($_GET['page']) && $_GET['page'] == 'mrtarget-client-page') {
        wp_redirect(admin_url('admin.php?page=mr-target-product-catalog'));
        exit;
    }
}

/**
 * 
 */
add_action('wp_ajax_handleSync', 'handleSync');
add_action('wp_ajax_nopriv_handleSync', 'handleSync');
function handleSync()
{
    //check_ajax_referer('handleSync', 'nonce');

    $productId = isset($_POST['productId']) ? intval($_POST['productId']) : 0;
    $status = isset($_POST['status']) ? sanitize_text_field($_POST['status']) : '';

    if (!$productId || !$status || !in_array($status, array('yes', 'no'))) {
        wp_send_json_error('Invalid parameters');
    }

    update_post_meta($productId, '_mrtarget_sync_activated', $status);

    wp_send_json_success('Sync status updated');
    wp_die();
}



add_action('wp_ajax_handleImport', 'handleImport');
function handleImport()
{
    //check_ajax_referer('handleImport', 'nonce');

    $sku = isset($_POST['sku']) ? sanitize_text_field($_POST['sku']) : '';

    if (empty($sku)) {
        wp_send_json_error('Invalid parameters');
    }

    $api_url = MR_TARGET_SYNC_URL . $sku;
    $response = wp_remote_get($api_url);
    //error_log(print_r($response,  true) . PHP_EOL, 3, wp_get_upload_dir()['basedir'] . '/external_data.log');

    if (is_wp_error($response)) {
        wp_send_json_error('Failed to retrieve data from API');
    }

    $data = wp_remote_retrieve_body($response);
    $product_data = json_decode($data, true);
    //error_log(print_r($product_data,  true) . PHP_EOL, 3, wp_get_upload_dir()['basedir'] . '/external_data.log');


    $new_product = array(
        'post_title'    => $product_data['name'],
        'post_content'  => $product_data['description'],
        'post_status'   => 'publish',
        'post_type'     => 'product',
        'meta_input'    => array(
            '_price'        => $product_data['price'],
            '_regular_price' => $product_data['regular_price'],
            '_sale_price'   => $product_data['sale_price'],
            '_stock'        => $product_data['stock_quantity'],
            '_weight'       => $product_data['weight'],
            '_length'       => $product_data['length'],
            '_width'        => $product_data['width'],
            '_height'       => $product_data['height'],
            '_sku'          => $sku,
        ),
    );

    $product_id = wp_insert_post($new_product);

    if (is_wp_error($product_id)) {
        wp_send_json_error('Failed to create new product');
    }

    if (!empty($product_data['categories'])) {
        wp_set_object_terms($product_id, $product_data['categories'], 'product_cat');
    }

    if (!empty($product_data['tags'])) {
        wp_set_object_terms($product_id, $product_data['tags'], 'product_tag');
    }

    if (!empty($product_data['featured_image'])) {
        $image_url = $product_data['featured_image'];
        $image_name = basename($image_url);
        $image_data = file_get_contents($image_url);
        $upload_dir = wp_upload_dir();
        $upload_path = $upload_dir['path'] . '/' . $image_name;
        file_put_contents($upload_path, $image_data);

        $file_array = array(
            'name'     => $image_name,
            'type'     => 'image/jpeg',
            'tmp_name' => $upload_path,
            'error'    => 0,
            'size'     => filesize($upload_path),
        );

        $attachment_id = media_handle_sideload($file_array, $product_id, $product_data['featured_image']);
        if (!is_wp_error($attachment_id)) {
            set_post_thumbnail($product_id, $attachment_id);
        }
    }
    if (!empty($product_data['gallery_images'])) {
        $image_urls = $product_data['gallery_images'];
        $gallery_image_ids = [];

        foreach ($image_urls as $image_url) {
            $image_data = file_get_contents($image_url);

            if ($image_data !== false) {
                $upload_dir = wp_upload_dir();
                $filename = basename($image_url);
                $file_path = $upload_dir['path'] . '/' . $filename;

                file_put_contents($file_path, $image_data);

                $wp_filetype = wp_check_filetype($filename, null);
                $attachment = array(
                    'post_mime_type' => $wp_filetype['type'],
                    'post_title' => sanitize_file_name($filename),
                    'post_content' => '',
                    'post_status' => 'inherit'
                );

                $attach_id = wp_insert_attachment($attachment, $file_path);

                require_once(ABSPATH . 'wp-admin/includes/image.php');
                $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
                wp_update_attachment_metadata($attach_id, $attach_data);

                // Add the image ID to the gallery array
                $gallery_image_ids[] = $attach_id;
            }
        }

        // Update the product gallery meta field with the image IDs
        if (!empty($gallery_image_ids)) {
            update_post_meta($product_id, '_product_image_gallery', implode(',', $gallery_image_ids));
        }
        $product = wc_get_product($product_id);
        $product->set_sku($sku);
        $product->save();
    }
    //
    if (!empty($product_data['attributes'])) {
        $product_attributes = array();
        foreach ($product_data['attributes'] as $attribute) {
            //$attribute_name = wc_attribute_taxonomy_name($attribute['name']);
            //$attribute_name = $attribute['name'];
            $attribute_name = wc_sanitize_taxonomy_name($attribute['name']);
            $attribute_label = $attribute['name'];

            if (!taxonomy_exists($attribute_name)) {
                register_taxonomy(
                    $attribute_name,
                    'product',
                    array(
                        'label'        => $attribute_label,
                        'public'       => true,
                        'show_ui'      => false,
                        'query_var'    => true,
                        'rewrite'      => false,
                        'hierarchical' => true,
                    )
                );
            }

            if ($attribute['is_taxonomy']) {
                wp_set_object_terms($product_id, $attribute['value'], $attribute_name);
                $product_attributes[$attribute_name] = array(
                    'name' => $attribute_label,
                    'value' => '',
                    'position' => 0,
                    'is_visible' => 1,
                    'is_variation' => 1,
                    'is_taxonomy' => 1,
                );
            } else {
                $product_attributes[$attribute_name] = array(
                    'name' => $attribute_label,
                    'value' => implode(' | ', $attribute['value']),
                    'position' => 0,
                    'is_visible' => 1,
                    'is_variation' => 1,
                    'is_taxonomy' => 0,
                );
            }
        }

        update_post_meta($product_id, '_product_attributes', $product_attributes);
    }


    if (!empty($product_data['variations'])) {
        wp_set_object_terms($product_id, 'variable', 'product_type');
        foreach ($product_data['variations'] as $variation_data) {
            $variation_post = array(
                'post_title'  => $product_data['name'] . ' - Variation',
                'post_name'   => 'product-' . $product_id . '-variation',
                'post_status' => 'publish',
                'post_parent' => $product_id,
                'post_type'   => 'product_variation',
                'guid'        => home_url('/?product_variation=' . $product_id),
            );

            $variation_id = wp_insert_post($variation_post);

            update_post_meta($variation_id, '_sku', $variation_data['sku']);
            update_post_meta($variation_id, '_price', $variation_data['price']);
            update_post_meta($variation_id, '_regular_price', $variation_data['regular_price']);
            update_post_meta($variation_id, '_sale_price', $variation_data['sale_price']);
            update_post_meta($variation_id, '_stock', $variation_data['stock_quantity']);

            foreach ($variation_data['attributes'] as $attribute_name => $attribute_value) {
                //$taxonomy = wc_attribute_taxonomy_name($attribute_name);
                $taxonomy = $attribute_name;
                update_post_meta($variation_id, 'attribute_' . $taxonomy, $attribute_value);
            }
        }
    }


    wp_send_json_success('Product imported successfully');
    wp_die();
}
