<?php
if (!defined('ABSPATH')) {
    exit;
}

$response = wp_remote_get(MR_TARGET_API_PRODUCTS_URL);

if (is_wp_error($response)) {
    echo '<p>' . __('Failed to retrieve data.', 'mrtargettextdomain') . '</p>';
    return;
}

$data = wp_remote_retrieve_body($response);
$products = json_decode($data, true);

if (empty($products)) {
    echo '<p>' . __('No products found.', 'mrtargettextdomain') . '</p>';
    return;
}
?>
<table class="widefat fixed" cellspacing="0">
    <thead>
        <tr>
            <th scope="col"><?php _e('Title', 'mrtargettextdomain'); ?></th>
            <th scope="col"><?php _e('SKU', 'mrtargettextdomain'); ?></th>
            <th scope="col"><?php _e('SKU Compliance', 'mrtargettextdomain'); ?></th>
            <th scope="col"><?php _e('Action', 'mrtargettextdomain'); ?></th>
        </tr>
    </thead>
    <tbody>
        <?php foreach ($products as $product) : ?>
            <tr>
                <td><?php echo esc_html($product['title']); ?></td>
                <td><?php echo esc_html($product['sku']); ?></td>
                <td>
                    <?php
                    $product_id = wc_get_product_id_by_sku($product['sku']);
                    $sync_activated = get_post_meta($product_id, '_mrtarget_sync_activated', true);
                    if ($product_id) {
                        echo __('SKU already exists', 'mrtargettextdomain');
                    } else {
                        echo __('SKU does not exist', 'mrtargettextdomain');
                    }
                    ?>
                </td>
                <td>
                    <?php
                    if ($product_id) {
                        if ($sync_activated === 'yes') {
                            echo '<button class="button button-secondary unSyncProduct" data-product-id="' . $product_id . '">' . __('Unsync', 'mrtargettextdomain') . '</button>';
                        } else {
                            echo '<button class="button button-primary syncProduct" data-product-id="' . $product_id . '">' . __('Sync', 'mrtargettextdomain') . '</button>';
                        }
                    } else {
                        echo '<button class="button button-primary importProduct" data-sku="' . $product['sku'] . '">' . __('Import', 'mrtargettextdomain') . '</button>';
                    }
                    ?>
                </td>
            </tr>
        <?php endforeach; ?>
    </tbody>
</table>
<script>
    document.addEventListener('DOMContentLoaded', function() {
        var unSyncButtons = document.querySelectorAll('.unSyncProduct');
        var syncButtons = document.querySelectorAll('.syncProduct');
        var importButtons = document.querySelectorAll('.importProduct');

        unSyncButtons.forEach(function(button) {
            button.addEventListener('click', function() {
                var newDiv = document.createElement('div');
                newDiv.classList.add('loader-container');
                var innerDiv = document.createElement('div');
                innerDiv.classList.add('loader');
                newDiv.appendChild(innerDiv);
                this.parentNode.insertBefore(newDiv, this.nextSibling);
                this.style.display = 'none';
                document.body.style.pointerEvents = 'none';
                var productId = this.dataset.productId;
                handleSync(productId, 'no');
            });
        });

        syncButtons.forEach(function(button) {
            button.addEventListener('click', function() {
                var newDiv = document.createElement('div');
                newDiv.classList.add('loader-container');
                var innerDiv = document.createElement('div');
                innerDiv.classList.add('loader');
                newDiv.appendChild(innerDiv);
                this.parentNode.insertBefore(newDiv, this.nextSibling);
                this.style.display = 'none';
                document.body.style.pointerEvents = 'none';
                var productId = this.dataset.productId;
                handleSync(productId, 'yes');
            });
        });

        importButtons.forEach(function(button) {
            button.addEventListener('click', function() {
                var newDiv = document.createElement('div');
                newDiv.classList.add('loader-container');
                var innerDiv = document.createElement('div');
                innerDiv.classList.add('loader');
                newDiv.appendChild(innerDiv);
                this.parentNode.insertBefore(newDiv, this.nextSibling);
                this.style.display = 'none';
                document.body.style.pointerEvents = 'none';
                var sku = this.dataset.sku;
                handleImport(sku);
            });
        });

        function handleSync(productId, status) {
            var data = new FormData();
            data.append('action', 'handleSync');
            data.append('productId', productId);
            data.append('status', status);

            fetch(ajaxurl, {
                    method: 'POST',
                    body: data,
                    credentials: 'same-origin',
                    headers: new Headers({
                        'X-Requested-With': 'XMLHttpRequest',
                    }),
                })
                .then(function(response) {
                    return response.json();
                })
                .then(function(response) {
                    window.location.reload();
                })
                .catch(function(error) {
                    console.error('Error:', error);
                });
        }

        function handleImport(sku) {
            var data = new FormData();
            data.append('action', 'handleImport');
            data.append('sku', sku);

            fetch(ajaxurl, {
                    method: 'POST',
                    body: data,
                    credentials: 'same-origin',
                    headers: new Headers({
                        'X-Requested-With': 'XMLHttpRequest',
                    }),
                })
                .then(function(response) {
                    return response.json();
                })
                .then(function(response) {
                    if (response.success) {
                        window.location.reload();
                    } else {
                        console.error('Import failed:', response.data);
                    }
                })
                .catch(function(error) {
                    console.error('Error:', error);
                });
        }
    });
</script>